{
    This file is part of the Free Pascal/NewPascal run time library.
    Copyright (c) 2014 by Maciej Izak (hnb)
    member of the NewPascal development team (http://newpascal.org)

    Copyright(c) 2004-2018 DaThoX

    It contains the generics collections library

    See the file COPYING.FPC, included in this distribution,
    for details about the copyright.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

 **********************************************************************}

unit sparta_Generics.Strings;

{$mode objfpc}{$H+}

interface

resourcestring
  SArgumentOutOfRange = 'Argument out of range';
  SArgumentNilNode = 'Node is nil';
  SDuplicatesNotAllowed = 'Duplicates not allowed in dictionary';
  SCollectionInconsistency = 'Collection inconsistency';
  SCollectionDuplicate = 'Collection does not allow duplicates';
  SDictionaryKeyDoesNotExist = 'Dictionary key does not exist';
  SItemNotFound = 'Item not found';

implementation

end.

