unit shlobjext; 

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;

const

 SFGAO_CANRENAME=$00000010;
 SFGAO_CANDELETE=$00000020;
 SFGAO_HASPROPSHEET =$00000040;
 SFGAO_DROPTARGET  =$00000100;
 SFGAO_CAPABILITYMASK  =$00000177;
 SFGAO_ISSLOW   = $00004000;
 SFGAO_GHOSTED	=$00008000;
 SFGAO_LINK=$00010000;
 SFGAO_SHARE=$00020000;
 SFGAO_READONLY	=$00040000;
 SFGAO_HIDDEN=$00080000;
 SFGAO_DISPLAYATTRMASK = SFGAO_ISSLOW or SFGAO_GHOSTED or SFGAO_LINK or SFGAO_SHARE or SFGAO_READONLY or SFGAO_HIDDEN;
 { (SFGAO_ISSLOW \
				 | SFGAO_GHOSTED \
				 | SFGAO_LINK \
				 | SFGAO_SHARE \
				 | SFGAO_READONLY \
				 | SFGAO_HIDDEN)
     }
 SFGAO_FILESYSANCESTOR =	$10000000;
 SFGAO_FOLDER	=	$20000000;
 SFGAO_FILESYSTEM =	$40000000;
 SFGAO_HASSUBFOLDER =	$80000000;
 SFGAO_CONTENTSMASK =	$80000000;
 SFGAO_VALIDATE	 =	$01000000;
 SFGAO_REMOVABLE =	$02000000;
 SFGAO_COMPRESSED =	$04000000;

implementation

end.

